'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _loreUtils = require('lore-utils');

var _loreReactForms = require('lore-react-forms');

var _loreReactFormsBootstrap = require('lore-react-forms-bootstrap');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (0, _createReactClass2.default)({
  displayName: 'Overlay',

  propTypes: {
    model: _propTypes2.default.object.isRequired,
    data: _propTypes2.default.object,
    validators: _propTypes2.default.oneOfType([_propTypes2.default.func, _propTypes2.default.object]),
    onChange: _propTypes2.default.func,
    onSubmit: _propTypes2.default.func,
    callbacks: _propTypes2.default.object,
    schema: _propTypes2.default.object.isRequired,
    fieldMap: _propTypes2.default.object.isRequired,
    actionMap: _propTypes2.default.object.isRequired,
    fields: _propTypes2.default.object.isRequired,
    actions: _propTypes2.default.array.isRequired
  },

  getInitialState: function getInitialState() {
    var model = this.props.model;


    return {
      data: _lodash2.default.merge({}, model.data),
      isSaving: false,
      showSuccessMessage: false,
      hasError: false,
      request: null
    };
  },

  request: function request(data) {
    var modelName = this.props.modelName;
    var _props = this.props,
        model = _props.model,
        request = _props.request;

    return request ? request(data) : lore.actions[modelName].destroy(model).payload;
  },

  onSubmit: function onSubmit() {
    var data = this.state.data;


    this.setState({
      isSaving: true,
      showSuccessMessage: false,
      hasError: false,
      request: this.request(data)
    });
  },

  onCancel: function onCancel(data) {
    this.props.onCancel(data);
  },

  onRequestSuccess: function onRequestSuccess(request) {
    this.setState({
      isSaving: false,
      request: request,
      showSuccessMessage: true,
      hasError: false
    });
    this.onCancel();
  },

  onRequestError: function onRequestError(request) {
    this.setState({
      isSaving: false,
      request: request,
      hasError: true
    });
  },

  onChange: function onChange(name, value) {
    var nextData = _lodash2.default.merge({}, this.state.data);
    nextData[name] = value;
    this.setState({
      data: nextData
    });
  },

  getValidators: function getValidators(data) {
    var validators = this.props.validators;

    return (0, _loreUtils.result)(validators || {}, data);
  },

  render: function render() {
    var modelName = this.props.modelName;
    var _props2 = this.props,
        _props2$title = _props2.title,
        title = _props2$title === undefined ? 'Delete ' + _lodash2.default.capitalize(modelName) : _props2$title,
        _props2$description = _props2.description,
        description = _props2$description === undefined ? '' : _props2$description,
        _props2$successMessag = _props2.successMessage,
        successMessage = _props2$successMessag === undefined ? _lodash2.default.capitalize(modelName) + ' deleted.' : _props2$successMessag,
        schema = _props2.schema,
        fieldMap = _props2.fieldMap,
        actionMap = _props2.actionMap,
        fields = _props2.fields,
        actions = _props2.actions;
    var _state = this.state,
        data = _state.data,
        request = _state.request,
        isSaving = _state.isSaving,
        showSuccessMessage = _state.showSuccessMessage,
        hasError = _state.hasError;


    var validators = this.getValidators(data);

    var requestProps = {
      request: request,
      reducer: modelName,
      onSuccess: this.onRequestSuccess,
      onError: this.onRequestError
    };

    return _react2.default.createElement(
      'div',
      { className: 'modal-dialog' },
      _react2.default.createElement(
        'div',
        { className: 'modal-content' },
        _react2.default.createElement(
          _loreReactFormsBootstrap.Overlay,
          { isVisible: isSaving },
          _react2.default.createElement(
            'div',
            null,
            _react2.default.createElement(
              'div',
              { className: 'modal-header' },
              _react2.default.createElement(
                'button',
                { type: 'button', className: 'close', onClick: this.onCancel },
                _react2.default.createElement(
                  'span',
                  null,
                  '\xD7'
                )
              ),
              title ? _react2.default.createElement(
                'h4',
                { className: 'modal-title' },
                title
              ) : null,
              description ? _react2.default.createElement(
                'p',
                { className: 'help-block' },
                description
              ) : null
            ),
            request && isSaving ? _react2.default.createElement(_loreReactFormsBootstrap.Request, requestProps) : null,
            showSuccessMessage ? _react2.default.createElement(_loreReactFormsBootstrap.SuccessMessage, {
              title: 'Success!',
              message: successMessage
            }) : null,
            hasError ? _react2.default.createElement(
              _loreReactFormsBootstrap.RequestError,
              { request: request },
              function (request) {
                return request.error;
              }
            ) : null,
            _react2.default.createElement(_loreReactForms.SchemaForm, {
              data: data,
              validators: validators,
              onChange: this.onChange,
              callbacks: {
                onSubmit: this.onSubmit,
                onCancel: this.onCancel
              },
              schema: schema,
              fieldMap: fieldMap,
              actionMap: actionMap,
              fields: fields || {},
              actions: actions || []
            })
          )
        )
      )
    );
  }

});
module.exports = exports['default'];